#include <msp.h>
#include "uart.h"
#include "fifo_queue.h"
#include "fifo_function_queue.h"
#include "common.h"

//Reception and Transmission Queues
static char rx_buffer[QUEUE_DIMENSION_UART];
static char tx_buffer[QUEUE_DIMENSION_UART];
static char tail_rx, head_rx;
static char tail_tx, head_tx;

//Callback function for queue
void (*message_received_callback)();

void initializeUART()
{
    //Keep module in reset while being configured
    //UCA0CTL1 = UCSWRST;
    EUSCI_A0 -> CTLW0 = EUSCI_A_CTLW0_SWRST;

    //Parity disabled, LSB first, 8-bit length, 1 stop bit, asynchronous uart mode
    //ACLK
    EUSCI_A0 -> CTLW0 |= EUSCI_A_CTLW0_UCSSEL_1;


    //UCA0CTL0 = 0;
    //UCA0CTL1 |= UCSSEL0;




    //Baud Rate = 9600bps, sourced by ACLK @ 32kHZ
    //N = frequency of brclk / desired baud rate
    //UCBR = int(N)
    //UCBRS = round((N-int(N))*8)
    UCA0BR0 = 3;
    UCA0BR1 = 0;

    //Second modulation stage = UCBRSx, Oversampling disabled
    //UCA0MCTL = UCBRS1 | UCBRS0;






    //Configure shared i/o pins
    //P1.2 Rx as input pin (P1SEL1 = 0, P1SEL0 = 1)
    //P1.3 Tx as output pin (P1SEL1 = 0, P1SEL0 = 1)
    P1DIR |= BIT3;
    P1SEL0 |= BIT2 | BIT3;

    //Init queues
    initQueue(&head_rx, &tail_rx);
    initQueue(&head_tx, &tail_tx);

    //Turn on the module
    //UCA0CTL1 &= ~UCSWRST;
    EUSCI_A0 -> CTLW1 &= ~EUSCI_A_CTLW0_SWRST;

    //Enable Interrupts
    //UC0IE = UCA0RXIE;
    //NVIC_SetPriority(PORT1_IRQn,3);
    //NVIC_EnableIRQ(PORT1_IRQn); // Enable PORT1 Interrupt

    EUSCI_A0 -> IE |= EUSCI_A_IE_RXIE;
    NVIC_SetPriority(EUSCIA0_IRQn,3);
    NVIC_EnableIRQ(EUSCIA0_IRQn);
}

void setUartCallbackFunction(void *callback_function)
{
    message_received_callback = callback_function;
}

void uartReceive(char* buffer_pointer)
{
    int i = 0;
    char value;

    if(!queueIsEmpty(head_rx, tail_rx))
    {
        value = getFromQueue(&tail_rx, rx_buffer);

        while (value != '\r')       //'\r' is expected as defined by the assignment
        {
            *(buffer_pointer + i) = value;

            if(!queueIsEmpty(head_rx, tail_rx))
            {
                value = getFromQueue(&tail_rx, rx_buffer);
            }
            else
            {
                value = 0;
            }

            i++;
        }
        *(buffer_pointer + i) = value;      //Copying '\r'
    }
}

void uartTransmit(char* buffer_pointer)
{
    int i = 0;

    while (*(buffer_pointer + i)  != '\r')  //'\r' is expected as defined by the assignment
    {
        if(!queueIsFull(head_tx, tail_tx))
        {
            addToQueue(*(buffer_pointer + i), &head_tx, tx_buffer);
        }
        i++;
    }

    if(!queueIsFull(head_tx, tail_tx))
    {
        addToQueue(*(buffer_pointer + i), &head_tx, tx_buffer);     //Adding '\r'
    }

    EUSCI_A0 -> IE |= EUSCI_A_IE_TXIE;
}

void EUSCIA0_IRQHandler(void)
{
    if(EUSCI_A0 -> IFG & EUSCI_A_IFG_RXIFG)
    {
        //ISR - Rx
        if(!queueIsFull(head_rx, tail_rx))
        {
            addToQueue(EUSCI_A0 -> RXBUF, &head_rx, rx_buffer);

            if(UCA0RXBUF  == '\r')
            {
                addToFunctionQueue(message_received_callback);
                __low_power_mode_off_on_exit();
            }
        }
    }
    else if(EUSCI_A0 -> IFG & EUSCI_A_IFG_TXIFG)
    {
        //ISR - Tx
        if(!queueIsEmpty(head_tx, tail_tx))
        {
            EUSCI_A0 -> TXBUF = getFromQueue(&tail_tx, tx_buffer);
        }
        else
        {
            EUSCI_A0 -> IE &= ~EUSCI_A_IE_TXIE;
        }
    }
}
